
struct FoRWrapper{T,F,G,C}
  arglen::Int64 
  gradhash::Vector{UInt64}
  hesshash::Vector{UInt64}
  buf_g::Vector{T}
  buf_h::Matrix{T}
  tapes::Dict{Type, ReverseDiff.CompiledTape}
  fun::F
  grad_withtape::G
  cfg::C
end


function FoRWrapper(f::F, sample_arg::Vector{T}, 
                    chunksize=min(length(sample_arg),16)) where{F,T}
  # Allocate everything:
  n = length(sample_arg)
  gradhash = [hash(NaN)]
  hesshash = [hash(NaN)]
  buf_g    = Array{T}(undef, n)
  buf_h    = Array{T}(undef, n, n)
  tapes    = Dict{Type,ReverseDiff.CompiledTape}()
  # The key of the idea here: create a closure that is an in-place gradient that
  # checks for compiled gradient tapes, and if it doesn't have one it generates
  # it and then uses that. Then the next time you call, it has that tape handy.
  grad_withtape = (g,p) -> begin
    if haskey(tapes, typeof(p))
      tape = tapes[typeof(p)]
    else
      newtape  = ReverseDiff.GradientTape(f, p)
      cnewtape = ReverseDiff.compile(newtape)
      tapes[typeof(p)] = cnewtape
      tape = cnewtape
    end
    ReverseDiff.gradient!(g, tape, p)
  end
  # A jacobian config to cut down on allocs in the final calling interface.
  cfg = ForwardDiff.JacobianConfig(grad_withtape, buf_g, sample_arg, 
                                   ForwardDiff.Chunk{chunksize}())
  FoRWrapper(n, gradhash, hesshash, buf_g, buf_h, tapes, f, grad_withtape, cfg)
end


function (M::FoRWrapper{T,F,G,C})(case::Symbol, arg::Vector{T}, 
                                  checkhash=true) where{T,F,G,C}
  # Check lengths:
  @assert length(arg) == M.arglen "Provided argument is not the correct length."
  # Check if we happen to have the desired grad or hessian sitting around:
  arghash = hash(arg)
  if checkhash
    ((case == :gradient) && (arghash === M.gradhash[1])) && return M.buf_g
    ((case == :hessian)  && (arghash === M.hesshash[1])) && return M.buf_h
  end
  # Now give back either value, gradient, or hessian, updating the cache value
  # appropriately for either one:
  if case == :value
    return M.fun(arg)
  elseif case == :gradient
    M.grad_withtape(M.buf_g, arg)
    M.gradhash[1] = arghash
    return M.buf_g
  elseif case == :hessian
    ForwardDiff.jacobian!(M.buf_h, M.grad_withtape, M.buf_g, arg, M.cfg)
    M.gradhash[1] = arghash
    M.hesshash[1] = arghash
    return M.buf_h
  else
    throw(error("Case options are :value, :gradient, or :hessian."))
  end
end

