
function ipopt_nlsolve(fn, ini; box_lower=-1e22, box_upper=1e22, kwargs...)
  # bring in some default arguments:
  optargs = merge(DEF_IPOPT_KWARGS, Dict(kwargs))
  # get the dimension of the problem:
  arglen  = length(ini)
  nconstr = arglen
  # set up box constraints for the args and, if applicable, the constraints:
  box_l = prepare_bounds(arglen, box_lower)
  box_u = prepare_bounds(arglen, box_upper)
  con_l = zeros(length(ini))
  con_u = zeros(length(ini))
  # function for the constraints:
  constr_eval = (p,store) -> fn(store, p)
  # "constraint" jacobian:
  con_buf = zeros(arglen)
  jac_buf = zeros(arglen, arglen)
  constr_jac = (p,r,c,v) -> begin 
    isnothing(v) && return jac_structure!(r, c, nconstr, nconstr) 
    ForwardDiff.jacobian!(jac_buf, fn, con_buf, p)
    v .= vec(jac_buf)
  end
  # Create the final problem:
  prob = CreateIpoptProblem(arglen,  box_l, box_u,        # arg count and box
                            nconstr, con_l, con_u,        # constr. count and box
                            arglen^2,                     # size of constr. jacobian
                            div(arglen*(arglen+1), 2),    # size of hessian ltri
                            _p->0.0,                      # objective
                            constr_eval,                  # constraint function
                            (p,g)->(g.=zeros(length(p))), # objective gradient
                            constr_jac,                   # constraint jacobian
                            dummy)                        # lagrangian hessian
  # Add all the options:
  ipopt_addoption!(prob, "hessian_approximation", "limited-memory")
  ipopt_addoption!(prob, "nlp_scaling_method",    "none")
  for (opt, val) in optargs
    ipopt_addoption!(prob, opt, val)
  end
  # Solve and return:
  prob.x = copy(ini)
  status = IpoptSolve(prob)
  (status=Ipopt._STATUS_CODES[status], minimizer=copy(prob.x))
end

